library wow_cache_manager;

import 'dart:io';

import 'package:flutter_cache_manager/flutter_cache_manager.dart';

part 'cache_manager/base_cache_manager.dart';
part 'cache_manager/image_cache_manager.dart';
part 'cache_manager/file_cache_manager.dart';
part 'cache_manager/video_cache_manager.dart';

class WowCacheManager {
  WowCacheManager._();

  factory WowCacheManager() => _instance;

  static final WowCacheManager _instance = WowCacheManager._();

  late final _ImageCacheManager _imageCacheManager;

  late final _FileCacheManager _fileCacheManager;

  late final _VideoCacheManager _videoCacheManager;

  /// Bắt buộc phải khởi tạo thông tin cho cache manager mỗi khi vào app
  static void initCacheInfo({
    required int maxCacheFile,
    required int maxDaysCacheFile,
    required int maxCacheImage,
    required int maxDaysCacheImage,
    required int maxCacheVideo,
    required int maxDaysCacheVideo,
  }) {
    _instance._imageCacheManager = _ImageCacheManager(
      stalePeriod: Duration(days: maxDaysCacheImage),
      maxNrOfCacheObjects: maxCacheImage,
    );
    _instance._fileCacheManager = _FileCacheManager(
      stalePeriod: Duration(days: maxDaysCacheFile),
      maxNrOfCacheObjects: maxCacheFile,
    );
    _instance._videoCacheManager = _VideoCacheManager(
      stalePeriod: Duration(days: maxDaysCacheVideo),
      maxNrOfCacheObjects: maxCacheVideo,
    );
  }

  /// Lấy file
  ///
  /// Nếu ko có file dưới cache thì tự động lưu cache
  static Future<File> getFile(String url) {
    return _instance._fileCacheManager.getCacheFile(url);
  }

  /// Xoá file cache
  static Future<void> removeFile(String url) {
    return _instance._fileCacheManager.removeFile(url);
  }

  /// Kiểm tra xem File có lưu cache chưa
  static Future<bool> isFileCached(String url) {
    return _instance._fileCacheManager.isCached(url);
  }

  /// Lưu file xuống cache
  static Future<File> saveFileToCache(String url) {
    return _instance._fileCacheManager.saveToCache(url);
  }

  /// Xoá tất cả cache file
  static Future<void> emptyCacheFile() {
    return _instance._fileCacheManager.emptyCache();
  }

  /// Lấy ảnh
  ///
  /// Nếu không có ảnh dưới cache thì tự động lưu cache
  static Future<File> getImage(String url) {
    return _instance._imageCacheManager.getCacheFile(url);
  }

  /// Xoá ảnh cache
  static Future<void> removeImage(String url) {
    return _instance._imageCacheManager.removeFile(url);
  }

  /// Kiểm tra xem ảnh đã lưu cache chưa
  static Future<bool> isImageCached(String url) {
    return _instance._imageCacheManager.isCached(url);
  }

  /// Lưu ảnh xuống cache
  static Future<File> saveImageToCache(String url) {
    return _instance._imageCacheManager.saveToCache(url);
  }

  /// Xoá tất cả cache hình ảnh
  static Future<void> emptyCacheImage() {
    return _instance._imageCacheManager.emptyCache();
  }

  /// Lấy video
  ///
  /// Nếu không có video dưới cache thì tự động lưu cache
  static Future<File> getVideo(String url) {
    return _instance._videoCacheManager.getCacheFile(url);
  }

  /// Xoá video cache
  static Future<void> removeVideo(String url) {
    return _instance._videoCacheManager.removeFile(url);
  }

  /// Kiểm tra video có lưu cache chưa
  static Future<bool> isVideoCached(String url) {
    return _instance._videoCacheManager.isCached(url);
  }

  /// Lưu video xuống cache
  static Future<File> saveVideoToCache(String url) {
    return _instance._videoCacheManager.saveToCache(url);
  }

  /// Xoá tất cả cache video
  static Future<void> emptyCacheVideo() {
    return _instance._videoCacheManager.emptyCache();
  }

  /// Xoá tất cả cache
  static Future<void> emptyCacheAll() {
    return Future.wait(
      [
        emptyCacheFile(),
        emptyCacheImage(),
        emptyCacheVideo(),
      ],
    );
  }
}
